@@ -19,10 +19,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
 #include <bonobo/bonobo-i18n.h>
 #include <gtk/gtkdialog.h>
 #include <libgnomedb/libgnomedb.h>
+#include <libgnomeui/gnome-client.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include "database-capplet.h"
 
@@ -44,6 +44,7 @@ create_dialog (gpointer user_data)
 					      NULL, 0,
 					      GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 					      NULL);
+	gtk_widget_set_usize (dialog, 450, 500);
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
 	gnome_window_icon_set_from_file (GTK_WINDOW (dialog),
 					 LIBGNOMEDB_ICONSDIR "/gnome-db.png");
@@ -60,11 +61,19 @@ create_dialog (gpointer user_data)
 int
 main (int argc, char *argv[])
 {
+	GnomeClient *session_client;
+
 	bindtextdomain (GETTEXT_PACKAGE, LIBGNOMEDB_LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
 	gnome_db_init ("gnome-database-properties", VERSION, argc, argv);
+
+	/* session management */
+	session_client = gnome_master_client ();
+	gnome_client_set_restart_style (session_client, GNOME_RESTART_NEVER);
+
+	/* application loop */
 	gnome_db_main_run ((GdaInitFunc) create_dialog, NULL);
 
 	return 0;
