@@ -136,8 +136,7 @@ provider_config_new (void)
 	/* add tip */
 	box = gnome_db_new_hbox_widget (FALSE, 3);
 	gtk_table_attach (GTK_TABLE (table), box, 0, 2, 1, 2,
-			  GTK_FILL | GTK_EXPAND | GTK_SHRINK,
-			  GTK_FILL | GTK_EXPAND | GTK_SHRINK, 5, 5);
+			  GTK_FILL, GTK_FILL, 5, 5);
 
 	button = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
 	gtk_widget_show (button);
