@@ -20,8 +20,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
-#include <libgda/gda-config.h>
+#include <string.h>
+#include <libgnomedb/gnome-db-config.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libgnomeui/gnome-druid-page-edge.h>
 #include <libgnomeui/gnome-druid-page-standard.h>
@@ -44,10 +44,12 @@ struct _GnomeDbDsnConfigDruidPrivate {
 	GtkWidget *general_provider;
 	GtkWidget *general_description;
 	GtkWidget *general_username;
+	GtkWidget *general_password;
 	GtkWidget *provider_page;
 	GtkWidget *provider_container;
 	GtkWidget *provider_detail;
 	GHashTable *provider_params;
+	GHashTable *remembered_values;
 	GtkWidget *end_page;
 };
 
@@ -117,6 +119,14 @@ remove_hash_func (gpointer key, gpointer value, gpointer user_data)
 	return TRUE;
 }
 
+static gboolean
+remove_remembered_hash (gpointer key, gpointer value, gpointer user_data)
+{
+	g_free (key);
+	g_free (value);
+	return TRUE;
+}
+
 /*
  * Callbacks
  */
@@ -139,7 +149,7 @@ druid_finished_cb (GnomeDruidPage *druid_page, GtkWidget *druid_widget, gpointer
 
 	/* set the internal dsn_info with the values of the widgets */
 	if (druid->priv->dsn_info)
-		gda_config_free_data_source_info (druid->priv->dsn_info);
+		gnome_db_config_free_data_source_info (druid->priv->dsn_info);
 
 	druid->priv->dsn_info = g_new0 (GdaDataSourceInfo, 1);
 	druid->priv->dsn_info->name =
@@ -152,6 +162,8 @@ druid_finished_cb (GnomeDruidPage *druid_page, GtkWidget *druid_widget, gpointer
 		g_strdup (gtk_entry_get_text (GTK_ENTRY (druid->priv->general_description)));
 	druid->priv->dsn_info->username =
 		g_strdup (gtk_entry_get_text (GTK_ENTRY (druid->priv->general_username)));
+	druid->priv->dsn_info->password =
+		g_strdup (gtk_entry_get_text (GTK_ENTRY (druid->priv->general_password)));
 
 	/* notify listeners */
 	g_signal_emit (G_OBJECT (druid), config_druid_signals[FINISHED], 0, FALSE);
@@ -208,10 +220,31 @@ general_next_pressed_cb (GnomeDruidPage *druid_page,
 
 		param = (const gchar *) l->data;
 		if (param) {
+			GHashTable *values;
+
 			box = gnome_db_new_hbox_widget (TRUE, 0);
 			label = gnome_db_new_label_widget (param);
 			gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
 			widget = gnome_db_new_entry_widget (0, TRUE);
+			if (!g_strcasecmp (param, "PASSWORD"))
+				gtk_entry_set_visibility (GTK_ENTRY (widget), FALSE);
+
+			/* set remembered value */
+			if ((values = g_hash_table_lookup (druid->priv->remembered_values, provider))) {
+				gtk_entry_set_text (GTK_ENTRY (widget),
+						    g_hash_table_lookup (values, param));
+			} else {
+				if (!g_strcasecmp (param, "USER")) {
+					gtk_entry_set_text (
+						GTK_ENTRY (widget),
+						gtk_entry_get_text (GTK_ENTRY (druid->priv->general_username)));
+				} else if (!g_strcasecmp (param, "PASSWORD")) {
+					gtk_entry_set_text (
+						GTK_ENTRY (widget),
+						gtk_entry_get_text (GTK_ENTRY (druid->priv->general_password)));
+				}
+			}
+
 			gtk_box_pack_start (GTK_BOX (box), widget, TRUE, TRUE, 0);
 			gtk_box_pack_start (GTK_BOX (druid->priv->provider_detail),
 					    box, TRUE, TRUE, 0);
@@ -230,6 +263,51 @@ general_next_pressed_cb (GnomeDruidPage *druid_page,
 	return FALSE;
 }
 
+static void
+add_remembered_hash (gpointer key, gpointer value, gpointer user_data)
+{
+	GHashTable *htable;
+	const gchar *provider;
+	const gchar *param_name = key;
+	const gchar *entry_value;
+	GtkWidget *entry = (GtkWidget *) value;
+	GnomeDbDsnConfigDruid *druid = (GnomeDbDsnConfigDruid *) user_data;
+
+	g_return_if_fail (GNOME_DB_IS_DSN_CONFIG_DRUID (druid));
+
+	provider = gnome_db_option_menu_get_selection (GTK_OPTION_MENU (druid->priv->general_provider));
+	entry_value = gtk_entry_get_text (GTK_ENTRY (entry));
+
+	htable = g_hash_table_lookup (druid->priv->remembered_values, provider);
+	if (htable) {
+		gpointer orig_key, orig_value;
+
+		if (g_hash_table_lookup_extended (htable, param_name, &orig_key, &orig_value)) {
+			g_hash_table_remove (htable, param_name);
+			g_free (orig_key);
+			g_free (orig_value);
+		}
+	} else {
+		htable = g_hash_table_new (g_str_hash, g_str_equal);
+		g_hash_table_insert (druid->priv->remembered_values, g_strdup (provider), htable);
+	}
+
+	g_hash_table_insert (htable, g_strdup (param_name), g_strdup (entry_value));
+}
+
+static gboolean
+provider_back_pressed_cb (GnomeDruidPage *druid_page,
+			  GtkWidget *druid_widget,
+			  gpointer user_data)
+{
+	GnomeDbDsnConfigDruid *druid = (GnomeDbDsnConfigDruid *) user_data;
+
+	g_return_val_if_fail (GNOME_DB_IS_DSN_CONFIG_DRUID (druid), TRUE);
+
+	g_hash_table_foreach (druid->priv->provider_params, (GHFunc) add_remembered_hash, druid);
+	return FALSE;
+}
+
 /*
  * GnomeDbDsnConfigDruid class implementation
  */
@@ -266,6 +344,7 @@ gnome_db_dsn_config_druid_init (GnomeDbDsnConfigDruid *druid,
 	druid->priv->dsn_info = g_new0 (GdaDataSourceInfo, 1);
 	druid->priv->provider_detail = NULL;
 	druid->priv->provider_params = g_hash_table_new (g_str_hash, g_str_equal);
+	druid->priv->remembered_values = g_hash_table_new (g_str_hash, g_str_equal);
 
 	/* load icons */
 	druid->priv->logo = gdk_pixbuf_new_from_file (LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
@@ -321,6 +400,12 @@ gnome_db_dsn_config_druid_init (GnomeDbDsnConfigDruid *druid,
 		GNOME_DRUID_PAGE_STANDARD (druid->priv->general_page),
 		_("Username"), druid->priv->general_username, NULL);
 
+	druid->priv->general_password = gnome_db_new_entry_widget (0, TRUE);
+	gtk_entry_set_visibility (GTK_ENTRY (druid->priv->general_password), FALSE);
+	gnome_druid_page_standard_append_item (
+		GNOME_DRUID_PAGE_STANDARD (druid->priv->general_page),
+		_("Password"), druid->priv->general_password, NULL);
+
 	gtk_widget_show (druid->priv->general_page);
 	gnome_druid_append_page (GNOME_DRUID (druid),
 				 GNOME_DRUID_PAGE (druid->priv->general_page));
@@ -354,6 +439,8 @@ gnome_db_dsn_config_druid_init (GnomeDbDsnConfigDruid *druid,
 
 	g_signal_connect (G_OBJECT (druid->priv->provider_page), "cancel",
 			  G_CALLBACK (druid_cancelled_cb), druid);
+	g_signal_connect (G_OBJECT (druid->priv->provider_page), "back",
+			  G_CALLBACK (provider_back_pressed_cb), druid);
 
 	/* end page */
 	druid->priv->end_page = gnome_druid_page_edge_new_with_vals (
@@ -380,7 +467,7 @@ gnome_db_dsn_config_druid_finalize (GObject *object)
 
 	/* free memory */
 	gdk_pixbuf_unref (druid->priv->logo);
-	gda_config_free_data_source_info (druid->priv->dsn_info);
+	gnome_db_config_free_data_source_info (druid->priv->dsn_info);
 
 	g_hash_table_foreach_remove (druid->priv->provider_params,
 				     (GHRFunc) remove_hash_func,
@@ -388,6 +475,12 @@ gnome_db_dsn_config_druid_finalize (GObject *object)
 	g_hash_table_destroy (druid->priv->provider_params);
 	druid->priv->provider_params = NULL;
 
+	g_hash_table_foreach_remove (druid->priv->remembered_values,
+				     (GHRFunc) remove_remembered_hash,
+				     NULL);
+	g_hash_table_destroy (druid->priv->remembered_values);
+	druid->priv->remembered_values = NULL;
+
 	g_free (druid->priv);
 	druid->priv = NULL;
 
