@@ -21,13 +21,15 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include <string.h>
 #include <libgda/gda-error.h>
 #include <libgnomedb/gnome-db-error.h>
 #include <libgnomedb/gnome-db-util.h>
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkentry.h>
+#include <gtk/gtkimage.h>
 #include <gtk/gtklabel.h>
+#include <gtk/gtkstock.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtktextview.h>
 #include <bonobo/bonobo-i18n.h>
@@ -116,7 +118,7 @@ detail_button_clicked_cb (GtkButton *button, gpointer user_data)
 					   error_widget->priv->height);
 		}
 	} else {
-		gtk_widget_show (error_widget->priv->detail_container);
+		gtk_widget_show_all (error_widget->priv->detail_container);
 		g_object_set (G_OBJECT (btn), "label", _("<< Hide detail"), NULL);
 		if (window){
 			gtk_window_get_size (window,
@@ -169,20 +171,34 @@ gnome_db_error_init (GnomeDbError *error_widget, GnomeDbErrorClass *klass)
 	GtkWidget *detail_table;
 	GtkWidget *label;
 	GtkWidget *scroll;
+	GtkWidget *image;
+	gchar *str;
 
 	error_widget->priv = g_new (GnomeDbErrorPrivate, 1);
 	error_widget->priv->current_list = NULL;
 	error_widget->priv->current_pos = -1;
 
 	/* create main container */
-	table = gnome_db_new_table_widget (3, 3, FALSE);
+	table = gnome_db_new_table_widget (4, 4, FALSE);
 	gtk_box_pack_start (GTK_BOX (error_widget), table, TRUE, TRUE, 0);
 
+	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_ERROR, GTK_ICON_SIZE_DIALOG);
+	gtk_widget_show (image);
+	gtk_table_attach (GTK_TABLE (table), image, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 3, 3);
+
+	label = gnome_db_new_label_widget ("");
+	gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
+	str = g_strdup_printf ("<b>%s</b>",
+			       _("An error has occurred in the underlying database"));
+	gtk_label_set_markup (GTK_LABEL (label), str);
+	g_free (str);
+	gtk_table_attach (GTK_TABLE (table), label, 1, 4, 0, 1, GTK_FILL, GTK_FILL, 3, 3);
+ 
 	label = gnome_db_new_label_widget (_("Error number"));
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 3, 3);
+	gtk_table_attach (GTK_TABLE (table), label, 1, 2, 1, 2, GTK_FILL, GTK_FILL, 3, 3);
 	error_widget->priv->error_number = gnome_db_new_entry_widget (0, FALSE);
 	gtk_table_attach (GTK_TABLE (table), error_widget->priv->error_number,
-			  1, 2, 0, 1, GTK_FILL, GTK_FILL, 3, 3);
+			  2, 3, 1, 2, GTK_FILL, GTK_FILL, 3, 3);
 
 	error_widget->priv->detail_button = gnome_db_new_button_widget (_("Show detail >>"));
 	gtk_signal_connect (GTK_OBJECT (error_widget->priv->detail_button),
@@ -190,12 +206,13 @@ gnome_db_error_init (GnomeDbError *error_widget, GnomeDbErrorClass *klass)
 			    GTK_SIGNAL_FUNC (detail_button_clicked_cb),
 			    error_widget);
 	gtk_table_attach (GTK_TABLE (table), error_widget->priv->detail_button,
-			  2, 3, 0, 1, GTK_FILL, GTK_FILL, 3, 3);
+			  3, 4, 1, 2, GTK_FILL, GTK_FILL, 3, 3);
 
 	label = gnome_db_new_label_widget (_("Description"));
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 3, 3);
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
+	gtk_table_attach (GTK_TABLE (table), label, 1, 2, 2, 3, GTK_FILL, GTK_FILL, 3, 3);
 	scroll = gnome_db_new_scrolled_window_widget ();
-	gtk_table_attach (GTK_TABLE (table), scroll, 1, 2, 1, 2,
+	gtk_table_attach (GTK_TABLE (table), scroll, 2, 3, 2, 3,
 			  GTK_FILL | GTK_EXPAND | GTK_SHRINK,
 			  GTK_FILL | GTK_EXPAND | GTK_SHRINK, 3, 3);
 	error_widget->priv->error_description = gnome_db_new_text_widget (NULL);
@@ -204,7 +221,7 @@ gnome_db_error_init (GnomeDbError *error_widget, GnomeDbErrorClass *klass)
 	/* create the detail container */
 	error_widget->priv->detail_container = gnome_db_new_frame_widget (_("Detail"));
 	gtk_table_attach (GTK_TABLE (table), error_widget->priv->detail_container,
-			  0, 3, 2, 3, GTK_FILL | GTK_SHRINK | GTK_EXPAND,
+			  0, 4, 3, 4, GTK_FILL | GTK_SHRINK | GTK_EXPAND,
 			  GTK_FILL | GTK_SHRINK | GTK_EXPAND, 3, 3);
 	detail_table = gnome_db_new_table_widget (2, 3, FALSE);
 	gtk_container_add (GTK_CONTAINER (error_widget->priv->detail_container), detail_table);
@@ -213,13 +230,13 @@ gnome_db_error_init (GnomeDbError *error_widget, GnomeDbErrorClass *klass)
 	gtk_table_attach (GTK_TABLE (detail_table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 3, 3);
 	error_widget->priv->error_source = gnome_db_new_entry_widget (0, FALSE);
 	gtk_table_attach (GTK_TABLE (detail_table), error_widget->priv->error_source,
-			  1, 2, 0, 1, GTK_FILL, GTK_FILL, 3, 3);
+			  1, 2, 0, 1, GTK_FILL | GTK_EXPAND | GTK_SHRINK, GTK_FILL, 3, 3);
 
 	label = gnome_db_new_label_widget (_("SQL state"));
 	gtk_table_attach (GTK_TABLE (detail_table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 3, 3);
 	error_widget->priv->error_sqlstate = gnome_db_new_entry_widget (0, FALSE);
 	gtk_table_attach (GTK_TABLE (detail_table), error_widget->priv->error_sqlstate,
-			  1, 2, 1, 2, GTK_FILL, GTK_FILL, 3, 3);
+			  1, 2, 1, 2, GTK_FILL | GTK_EXPAND | GTK_SHRINK, GTK_FILL, 3, 3);
 
 	gtk_widget_hide (error_widget->priv->detail_container);
 }
